//////////////////////////////////////////////////////////////////////////////////////
/*
Cellular Texture generation

Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

#include "CellularTxt.h"

namespace Math3D
{

Cellular::Cellular(int points, int width, int height, Vector3 color, int type)
{
	make(points, width, height, color, type);
}

Cellular::Cellular(const Cellular &cel)
{
	pPoints = NULL;
	pMap = NULL;
	*this = cel;
}

Cellular::Cellular()
{
	pMap = NULL;
	pPoints = NULL;
	width = 0;
	height = 0;
	numPoints = 0;
	color = Vector3(-1,-1,-1);
	type = 0;
}

Cellular::~Cellular()
{
	if (pMap)
		delete [] pMap;
	pMap = NULL;
	
	if (pPoints)
		delete [] pPoints;
	pPoints = NULL;
	
	width = 0;
	height = 0;
	numPoints = 0;
	texture = -1;
	color = Vector3(-1,-1,-1);
}

Cellular &Cellular::operator= (const Cellular &cel)
{	
	if (pPoints)
		delete [] pPoints;
	if (pMap)
		delete [] pMap;

	numPoints = cel.numPoints;
	width = cel.width;
	height = cel.height;
	texture = cel.texture;
	type = cel.type;
	color = cel.color;

	if (numPoints > 0 && cel.pPoints)
	{	
		pPoints = new Vector2[numPoints];
		memcpy((void*)pPoints,(void*)cel.pPoints,numPoints*sizeof(Vector2));

	}
	if (width > 0 && height > 0 && cel.pMap)
	{
		pMap = new char[3*width*height];
		memcpy((void*)pMap,(void*)cel.pMap,3*width*height);

	}
	
	return *this;
}

Cellular Cellular::operator+ (Cellular cel)
{
	Cellular tmp;
	if (width >= cel.width)
		tmp.width = width;
	else
		tmp.width = cel.width;
	
	if (height >= cel.height)
		tmp.height = height;
	else
		tmp.height = cel.height;
	
	tmp.pMap = new char[3*tmp.width*tmp.height];
	for (int i=0; i < 3*tmp.width*tmp.height; i++)
	{
		tmp.pMap[i] = pMap[i] + cel.pMap[i];
	}
	
	return tmp;
}

Cellular Cellular::operator- (Cellular cel)
{
	Cellular tmp;
	if (width >= cel.width)
		tmp.width = width;
	else
		tmp.width = cel.width;
	
	if (height >= cel.height)
		tmp.height = height;
	else
		tmp.height = cel.height;
	
	tmp.pMap = new char[3*tmp.width*tmp.height];	
	for (int i=0; i < 3*tmp.width*tmp.height; i++)
	{
		tmp.pMap[i] = pMap[i] - cel.pMap[i];
	}
	
	return tmp;
}

bool Cellular::make(int points, int width, int height, Vector3 color, int type)
{
	numPoints = points;
	this->width = width;
	this->height = height;
	this->color = color;
	
	pMap = new char[3*width*height];
	float distBuffer[width*height];
	pPoints =  new Vector2[points];
	
	Vector3 minDist = Vector3(1000000,1000000,1000000);
	Vector3 maxDist = Vector3(0,0,0);
	
	srand(1);
	for (int i=0; i < points; i++)
	{
		pPoints[i].x = rand() % width;
		pPoints[i].y = rand() % height;
	}
	
	// setup distance map
	for (int y=0; y < height; y++)
	{
		for (int x=0; x < width; x++)
		{
			Vector3 dist = getDistance(Vector2(x,y));
			//std::cout << "Dist: " << dist.x << "x" << dist.y << "x" << dist.z << '\n';
			switch (type)
			{
				case CEL_INV_DIST2_MINUS_DIST1:
				case CEL_DIST2_MINUS_DIST1:
					distBuffer[x + y*width] = dist.y - dist.x;
					if ((dist.y - dist.x) < minDist.x)
						minDist.x = dist.y - dist.x;
					if ((dist.y - dist.x) > maxDist.x)
						maxDist.x = dist.y - dist.x;
				break;
				case CEL_INV_DIST1_MUL_DIST2:
				case CEL_DIST1_MUL_DIST2:
					distBuffer[x + y*width] = dist.x * dist.y;
					if ((dist.x * dist.y) < minDist.x)
						minDist.x = dist.x * dist.y;
					if ((dist.x * dist.y) > maxDist.x)
						maxDist.x = dist.x * dist.y;
				break;
				case CEL_INV_SQUAREDIST:
				case CEL_SQUAREDIST:
				default:
					distBuffer[x + y*width] = dist.x * dist.x;
					if(dist.x < minDist.x)
						minDist.x = dist.x;
					if (dist.x > maxDist.x)
						maxDist.x = dist.x;
				break;
			}
		}
	}

	switch (type)
	{
		case CEL_INV_SQUAREDIST:
		case CEL_SQUAREDIST:
			minDist.x = minDist.x * minDist.x;
			maxDist.x = maxDist.x * maxDist.x;
		break;
		default:
		break;
	}
	
	// draw map
	for (int y=0; y < height; y++)
	{
		for (int x=0; x < 3*width; x += 3)
		{
			Vector3 col = color * ((distBuffer[x/3 + y*width] - minDist.x) / (maxDist.x - minDist.x) * 255);
			
			switch (type)
			{
				case CEL_INV_DIST2_MINUS_DIST1:
				case CEL_INV_DIST1_MUL_DIST2:
				case CEL_INV_SQUAREDIST:
					col =  Vector3(255,255,255) - col;
				break;
				default:
				break;
			}
			
			pMap[x + y*3*width] = col.x; 	// R
			pMap[x+1 + y*3*width] = col.y;	// G
			pMap[x+2 + y*3*width] = col.z;	// B
		}
	}
	
	return true;
}

UINT Cellular::createTexture()
{	
	if (width <= 0 || height <= 0 || !pMap)
		return -1;
		
	(TextureMan::instance()).createDataTexture(texture,pMap,width,height);
	return texture;
}

UINT Cellular::createTexture(int points, int width, int height, Vector3 color, int type)
{
	if (make(points,width,height,color,type))
		return createTexture();
	else
		return -1;
}

////////////////////////////////////////////////////////////////////////////////////

Vector3 Cellular::getDistance(Vector2 pos)
{
	Vector3 min = Vector3(100000,100000,100000);
	for (int i=0; i < numPoints; i++)
	{
		float dist = wrapDist(pos,pPoints[i]);//sqrt( (pPoints[i].x-pos.x)*(pPoints[i].x-pos.x) + (pPoints[i].y-pos.y)*(pPoints[i].y-pos.y) );
		if (dist < min.x) 
		{
			min.z = min.y;
			min.y = min.x;
			min.x = dist;
		}
		else if (dist < min.y)
		{
			min.z = min.y;
			min.y = dist;
		}
		else if (dist < min.z)
			min.z = dist;
	}
	
	return min;
}

float Cellular::wrapDist(Vector2 pos, Vector2 point)
{
	float dx = pos.x - point.x, dy = pos.y - point.y;
	
	// abs()
	if (dx < 0)
		dx = -dx;
	if (dy < 0)
		dy = -dy;
	
	if (dx > width/2) dx = width - dx;
	if (dy > height/2) dy = height - dy;
	
	return sqrt(dx*dx + dy*dy);
}

} // namespace
