//////////////////////////////////////////////////////////////////////////////////////
/*
DGUIControls - DrX GUI, a OGL GUI framework.

Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////


/////////////////// INCLUDE & DEFINE ///////////////////
#include "OglSdl.h"

namespace OglSdl
{

/////////////////// CLASS - OglSdl ///////////////////
OglSdl::OglSdl(long flag)
{
    if( SDL_Init(flag) < 0 )
    	throw ErrHand::Error("SDL_INIT() ERROR!",0);
}

OglSdl::~OglSdl()
{
    SDL_Quit();
}

bool OglSdl::createWindow(const char *caption, int width, int height, int depth)
{
	this->width = width;
	this->height = height;
	this->depth = depth;
	
	setupPixelFormat();
	
	sdlSurf = SDL_SetVideoMode(width, height, depth, VideoFlags);

	if( sdlSurf == NULL )
	{
		throw ErrHand::Error("SDL_SetVideoMode Faliure!",0);
	}

	//		Win caption,	icon caption
	SDL_WM_SetCaption(caption, caption);
	
	sizeOglScreen(width,height);
	
	return true;
}

void OglSdl::setFullScreen()
{
	if(SDL_WM_ToggleFullScreen(sdlSurf) == 0)
    		throw ErrHand::Error("SDL_WM_ToggleFullScreen Faliure!",1);
}

void OglSdl::resizeWindow(int width, int height)
{
	sdlSurf = SDL_SetVideoMode( width, height, depth, VideoFlags );
	sizeOglScreen(width, height);
	
	if(sdlSurf == NULL)
	{
		throw ErrHand::Error("Error Resizing window",0);
	}
}

void OglSdl::orthoMode(int left, int top, int right, int bottom)
{
	// Switch to our projection matrix so that we can change it to ortho mode, not perspective.
	glMatrixMode(GL_PROJECTION);						

	// Push on a new matrix so that we can just pop it off to go back to perspective mode
	glPushMatrix();									
	
	// Reset the current matrix to our identify matrix
	glLoadIdentity();								

	//Pass in our 2D ortho screen coordinates.like so (left, right, bottom, top).  The last
	// 2 parameters are the near and far planes.
	glOrtho( left, right, bottom, top, 0, 1 );	
	
	// Switch to model view so that we can render the scope image
	glMatrixMode(GL_MODELVIEW);								

	// Initialize the current model view matrix with the identity matrix
	glLoadIdentity();		
	glDisable(GL_DEPTH_TEST);
}

void OglSdl::orthoModeFullScreen()
{
	orthoMode(0,0,width,height);
}

///////////////////////////////// PERSPECTIVE MODE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function changes our returns our projection mode from 2D to 3D
/////
///////////////////////////////// PERSPECTIVE MODE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void OglSdl::perspectiveMode()										// Set Up A Perspective View
{
	// This function doesn't really go to perspective mode, since we aren't
	// using gluPerspective(), but if you look in OrthoMode(), we push on
	// a matrix once we change to our projection matrix.  All we need to do is
	// pop that matrix off of our stack and it will return to our original perspective
	// matrix.  So, before we pop the current matrix, we need to tell OpenGL which
	// matrix we want to use by changing our current matrix mode to perspective.
	// Then we can go back to our model view matrix and render normally.

	// Enter into our projection matrix mode
	glMatrixMode( GL_PROJECTION );							

	// Pop off the last matrix pushed on when in projection mode (Get rid of ortho mode)
	glPopMatrix();											

	// Go back to our model view matrix like normal
	glMatrixMode( GL_MODELVIEW );							
	glEnable(GL_DEPTH_TEST);
	// We should be in the normal 3D perspective mode now
}

//////////////////////////////////////////////////////////////////////

void OglSdl::setupPixelFormat()
{

	VideoFlags    = SDL_OPENGL;                            // it's an openGL window
	VideoFlags   |= SDL_HWPALETTE;                         // exclusive access to hardware colour palette
	VideoFlags   |= SDL_RESIZABLE;                         // the window must be resizeable

	const SDL_VideoInfo * VideoInfo = SDL_GetVideoInfo();  // query SDL for information about our video hardware

	if(VideoInfo == NULL)                                  // if this query fails
	{
		throw ErrHand::Error("Failed getting Video Info",0);
	}

	///////////// we set the system dependant flags here
	if(VideoInfo -> hw_available)                          // is it a hardware surface
		VideoFlags |= SDL_HWSURFACE;
	else
		VideoFlags |= SDL_SWSURFACE;
    
	// Blitting is fast copying / moving /swapping of contiguous sections of memory
	// for more about blitting check out : http://www.csc.liv.ac.uk/~fish/HTML/blitzman/bm_blitter.html
	if(VideoInfo -> blit_hw)                               // is hardware blitting available
		VideoFlags |= SDL_HWACCEL;

	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,   depth);
	SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);
	 // this and the next three lines set the bits allocated per pixel -
	SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);
	SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);      // - for the accumulation buffer to 0
	SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
	SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);
}

void OglSdl::sizeOglScreen(int width, int height)
{
	if (height==0)
	{
		height=1;
	}

	glViewport(0,0,width,height);

	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();

		// FOV        // Ratio                   //  The farthest distance before it stops drawing)
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 1 ,150.0f);
 
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

} // namespace
